@@ -25,6 +25,11 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/regexp.h>
 #include <the_Foundation/object.h>
 #include <the_Foundation/path.h>
+#include <the_Foundation/regexp.h>
+
+iRegExp *newGemtextLink_RegExp(void) {
+    return new_RegExp("=>\\s*([^\\s]+)(\\s.*)?", 0);
+}
 
 void init_Url(iUrl *d, const iString *text) {
     /* Handle "file:" as a special case since it only has the path part. */
@@ -38,7 +43,7 @@ void init_Url(iUrl *d, const iString *text) {
     static iRegExp *urlPattern_;
     static iRegExp *authPattern_;
     if (!urlPattern_) {
-        urlPattern_  = new_RegExp("^(([^:/?#]+):)?(//([^/?#]*))?"
+        urlPattern_  = new_RegExp("^(([-.+a-z0-9]+):)?(//([^/?#]*))?"
                                  "([^?#]*)(\\?([^#]*))?(#(.*))?",
                                  caseInsensitive_RegExpOption);
         authPattern_ = new_RegExp("(([^@]+)@)?(([^:\\[\\]]+)"
@@ -248,7 +253,7 @@ void urlEncodePath_String(iString *d) {
     iString *encoded = new_String();
     appendRange_String(encoded, (iRangecc){ constBegin_String(d), url.path.start });
     iString *path    = newRange_String(url.path);
-    iString *encPath = urlEncodeExclude_String(path, "%/ ");
+    iString *encPath = urlEncodeExclude_String(path, "%/= ");
     append_String(encoded, encPath);
     delete_String(encPath);
     delete_String(path);
@@ -327,6 +332,11 @@ const iString *absoluteUrl_String(const iString *d, const iString *urlMaybeRelat
             appendCStr_String(absolute, "/");
         }
         appendRange_String(absolute, rel.path);
+        /* If this is known to be a directory reference, append a slash. */
+        if (!endsWith_String(absolute, "/") &&
+            (equal_Rangecc(rel.path, "..") || endsWith_Rangecc(rel.path, "/.."))) {
+            appendCStr_String(absolute, "/");
+        }
     }
     else if (isDef_(rel.query)) {
         /* Just a new query. */
@@ -417,6 +427,87 @@ const char *makeFileUrl_CStr(const char *localFilePath) {
     return cstrCollect_String(makeFileUrl_String(collectNewCStr_String(localFilePath)));
 }
 
+iString *localFilePathFromUrl_String(const iString *d) {
+    iUrl url;
+    init_Url(&url, d);
+    if (!equalCase_Rangecc(url.scheme, "file")) {
+        return NULL;
+    }
+    iString *path = urlDecode_String(collect_String(newRange_String(url.path)));
+#if defined (iPlatformMsys)
+    /* Remove the extra slash from the beginning. */
+    if (startsWith_String(path, "/")) {
+        remove_Block(&path->chars, 0, 1);
+    }
+    replace_Block(&path->chars, '/', '\\');
+#endif
+    return path;
+}
+
+const iString *findContainerArchive_Path(const iString *path) {
+    iBeginCollect();
+    while (!isEmpty_String(path) && cmp_String(path, ".")) {
+        iString *dir = newRange_String(dirName_Path(path));
+        if (endsWithCase_String(dir, ".zip") ||
+            endsWithCase_String(dir, ".gpub")) {
+            iEndCollect();
+            return collect_String(dir);
+        }
+        path = collect_String(dir);
+    }
+    iEndCollect();
+    return NULL;
+}
+
+const char *mediaType_Path(const iString *path) {
+    if (endsWithCase_String(path, ".gmi") || endsWithCase_String(path, ".gemini")) {
+        return "text/gemini; charset=utf-8";
+    }
+    /* TODO: It would be better to default to text/plain, but switch to
+       application/octet-stream if the contents fail to parse as UTF-8. */
+    else if (endsWithCase_String(path, ".txt") ||
+             endsWithCase_String(path, ".md") ||
+             endsWithCase_String(path, ".c") ||
+             endsWithCase_String(path, ".h") ||
+             endsWithCase_String(path, ".cc") ||
+             endsWithCase_String(path, ".hh") ||
+             endsWithCase_String(path, ".cpp") ||
+             endsWithCase_String(path, ".hpp")) {
+        return "text/plain";
+    }
+    else if (endsWithCase_String(path, ".zip")) {
+        return "application/zip";
+    }
+    else if (endsWithCase_String(path, ".gpub")) {
+        return "application/gpub+zip";
+    }
+    else if (endsWithCase_String(path, ".xml")) {
+        return "text/xml";
+    }
+    else if (endsWithCase_String(path, ".png")) {
+        return "image/png";
+    }
+    else if (endsWithCase_String(path, ".jpg") || endsWithCase_String(path, ".jpeg")) {
+        return "image/jpeg";
+    }
+    else if (endsWithCase_String(path, ".gif")) {
+        return "image/gif";
+    }
+    else if (endsWithCase_String(path, ".wav")) {
+        return "audio/wave";
+    }
+    else if (endsWithCase_String(path, ".ogg")) {
+        return "audio/ogg";
+    }
+    else if (endsWithCase_String(path, ".mp3")) {
+        return "audio/mpeg";
+    }
+    else if (endsWithCase_String(path, ".mid")) {
+        return "audio/midi";
+    }
+    return "application/octet-stream";
+}
+
 void urlEncodeSpaces_String(iString *d) {
     for (;;) {
         const size_t pos = indexOfCStr_String(d, " ");
@@ -435,6 +526,12 @@ const iString *withSpacesEncoded_String(const iString *d) {
     return collect_String(enc);
 }
 
+iRangecc mediaTypeWithoutParameters_Rangecc(iRangecc mime) {
+    iRangecc part = iNullRange;
+    nextSplit_Rangecc(mime, ";", &part);
+    return part;
+}
+
 const iString *feedEntryOpenCommand_String(const iString *url, int newTab) {
     if (!isEmpty_String(url)) {
         iString *cmd = collectNew_String();
