@@ -26,6 +26,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/string.h>
 
 iDeclareType(GmError)
+iDeclareType(RegExp)
 iDeclareType(Url)
 
 /* Response status codes. */
@@ -90,6 +91,8 @@ struct Impl_GmError {
 iBool               isDefined_GmError   (enum iGmStatusCode code);
 const iGmError *    get_GmError         (enum iGmStatusCode code);
 
+iRegExp *       newGemtextLink_RegExp   (void);
+
 struct Impl_Url {
     iRangecc scheme;
     iRangecc host;
@@ -115,7 +118,14 @@ void            urlDecodePath_String    (iString *);
 void            urlEncodePath_String    (iString *);
 iString *       makeFileUrl_String      (const iString *localFilePath);
 const char *    makeFileUrl_CStr        (const char *localFilePath);
+iString *       localFilePathFromUrl_String(const iString *);
 void            urlEncodeSpaces_String  (iString *);
 const iString * withSpacesEncoded_String(const iString *);
 
+const char *    mediaType_Path                      (const iString *path);
+iRangecc        mediaTypeWithoutParameters_Rangecc  (iRangecc mime);
+
+const iString * findContainerArchive_Path           (const iString *path);
+
+
 const iString * feedEntryOpenCommand_String (const iString *url, int newTab); /* checks fragment */
