@@ -1,3 +1,25 @@
+/* Copyright 2021 Jaakko Keränen <jaakko.keranen@iki.fi>
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+
+1. Redistributions of source code must retain the above copyright notice, this
+   list of conditions and the following disclaimer.
+2. Redistributions in binary form must reproduce the above copyright notice,
+   this list of conditions and the following disclaimer in the documentation
+   and/or other materials provided with the distribution.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
+ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
+
 #include "lang.h"
 #include "embedded.h"
 
@@ -57,15 +79,18 @@ static void load_Lang_(iLang *d, const char *id) {
                        : equal_CStr(id, "ru")      ? &blobRu_Embedded
                        : equal_CStr(id, "es")      ? &blobEs_Embedded
                        : equal_CStr(id, "de")      ? &blobDe_Embedded
+                       : equal_CStr(id, "ia")      ? &blobIa_Embedded
                        : equal_CStr(id, "ie")      ? &blobIe_Embedded
                        : equal_CStr(id, "sr")      ? &blobSr_Embedded
+                       : equal_CStr(id, "tok")     ? &blobTok_Embedded
                        : equal_CStr(id, "zh_Hans") ? &blobZh_Hans_Embedded
                        : equal_CStr(id, "zh_Hant") ? &blobZh_Hant_Embedded
                                                    : &blobEn_Embedded;
     if (data == &blobRu_Embedded || data == &blobSr_Embedded) {
         d->pluralType = slavic_PluralType;
     }
-    else if (data == &blobZh_Hans_Embedded || data == &blobZh_Hant_Embedded) {
+    else if (data == &blobZh_Hans_Embedded || data == &blobZh_Hant_Embedded ||
+             data == &blobTok_Embedded) {
         d->pluralType = none_PluralType;
     }
     else {
