@@ -106,7 +106,8 @@ struct Impl_SidebarWidget {
     size_t            numUnreadEntries;
     iWidget *         resizer;
     iWidget *         menu;
-    iSidebarItem *    contextItem; /* list item accessed in the context menu */
+    iSidebarItem *    contextItem;  /* list item accessed in the context menu */
+    size_t            contextIndex; /* index of list item accessed in the context menu */
 };
 
 iDefineObjectConstructionArgs(SidebarWidget, (enum iSidebarSide side), side)
@@ -373,10 +374,6 @@ static void updateItems_SidebarWidget_(iSidebarWidget *d) {
             break;
         }
         case identities_SidebarMode: {
-            /* Actions. */ {
-                addActionButton_SidebarWidget_(d, add_Icon " ${sidebar.action.ident.new}", "ident.new", 0);
-                addActionButton_SidebarWidget_(d, "${sidebar.action.ident.import}", "ident.import", 0);
-            }
             const iString *tabUrl = url_DocumentWidget(document_App());
             isEmpty = iTrue;
             iConstForEach(PtrArray, i, identities_GmCerts(certs_App())) {
@@ -413,6 +410,11 @@ static void updateItems_SidebarWidget_(iSidebarWidget *d) {
                 iRelease(item);
                 isEmpty = iFalse;
             }
+            /* Actions. */
+            if (!isEmpty) {
+                addActionButton_SidebarWidget_(d, add_Icon " ${sidebar.action.ident.new}", "ident.new", 0);
+                addActionButton_SidebarWidget_(d, "${sidebar.action.ident.import}", "ident.import", 0);
+            }
             const iMenuItem menuItems[] = {
                 { person_Icon " ${ident.use}", 0, 0, "ident.use arg:1" },
                 { close_Icon " ${ident.stopuse}", 0, 0, "ident.use arg:0" },
@@ -644,6 +646,7 @@ void init_SidebarWidget(iSidebarWidget *d, enum iSidebarSide side) {
                  "actions");
     setBackgroundColor_Widget(d->actions, uiBackgroundSidebar_ColorId);
     d->contextItem = NULL;
+    d->contextIndex = iInvalidPos;
     d->blank = new_Widget();
     addChildFlags_Widget(content, iClob(d->blank), resizeChildren_WidgetFlag);
     addChildFlags_Widget(vdiv, iClob(content), expand_WidgetFlag);
@@ -844,7 +847,6 @@ iBool handleBookmarkEditorCommands_SidebarWidget_(iWidget *editor, const char *c
                                     isSelected_Widget(findChild_Widget(editor, "bmed.tag.linksplit")));
             postCommand_App("bookmarks.changed");
         }
-        setFlags_Widget(as_Widget(d), disabled_WidgetFlag, iFalse);
         destroy_Widget(editor);
         return iTrue;
     }
@@ -985,11 +987,8 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
             itemClicked_SidebarWidget_(d, pointerLabel_Command(cmd, "item"));
             return iTrue;
         }
-        else if (isCommand_Widget(w, ev, "menu.opened")) {
-            setFlags_Widget(as_Widget(d->list), disabled_WidgetFlag, iTrue);
-        }
         else if (isCommand_Widget(w, ev, "menu.closed")) {
-            setFlags_Widget(as_Widget(d->list), disabled_WidgetFlag, iFalse);
+         //   invalidateItem_ListWidget(d->list, d->contextIndex);
         }
         else if (isCommand_Widget(w, ev, "bookmark.open")) {
             const iSidebarItem *item = d->contextItem;
@@ -1010,7 +1009,6 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
         else if (isCommand_Widget(w, ev, "bookmark.edit")) {
             const iSidebarItem *item = d->contextItem;
             if (d->mode == bookmarks_SidebarMode && item) {
-                setFlags_Widget(w, disabled_WidgetFlag, iTrue);
                 iWidget *dlg = makeBookmarkEditor_Widget();
                 setId_Widget(dlg, format_CStr("bmed.%s", cstr_String(id_Widget(w))));
                 iBookmark *bm = get_Bookmarks(bookmarks_App(), item->id);
@@ -1038,7 +1036,6 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
         else if (isCommand_Widget(w, ev, "bookmark.dup")) {
             const iSidebarItem *item = d->contextItem;
             if (d->mode == bookmarks_SidebarMode && item) {
-                setFlags_Widget(w, disabled_WidgetFlag, iTrue);
                 iBookmark *bm = get_Bookmarks(bookmarks_App(), item->id);
                 const iBool isRemote = hasTag_Bookmark(bm, remote_BookmarkTag);
                 iChar icon = isRemote ? 0x1f588 : bm->icon;
@@ -1131,7 +1128,6 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
                         return iTrue;
                     }
                     if (isCommand_Widget(w, ev, "feed.entry.edit")) {
-                        setFlags_Widget(w, disabled_WidgetFlag, iTrue);
                         makeFeedSettings_Widget(id_Bookmark(feedBookmark));
                         return iTrue;
                     }
@@ -1307,6 +1303,10 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
                 setCursor_Window(get_Window(), SDL_SYSTEM_CURSOR_ARROW);
             }
         }
+        if (d->contextIndex != iInvalidPos) {
+            invalidateItem_ListWidget(d->list, d->contextIndex);
+            d->contextIndex = iInvalidPos;
+        }
     }
     if (d->menu && ev->type == SDL_MOUSEBUTTONDOWN) {
         if (ev->button.button == SDL_BUTTON_RIGHT) {
@@ -1315,7 +1315,12 @@ static iBool processEvent_SidebarWidget_(iSidebarWidget *d, const SDL_Event *ev)
                 updateMouseHover_ListWidget(d->list);
             }
             if (constHoverItem_ListWidget(d->list) || isVisible_Widget(d->menu)) {
-                d->contextItem = hoverItem_ListWidget(d->list);
+                d->contextItem  = hoverItem_ListWidget(d->list);
+                /* Context is drawn in hover state. */
+                if (d->contextIndex != iInvalidPos) {
+                    invalidateItem_ListWidget(d->list, d->contextIndex);
+                }
+                d->contextIndex = hoverItemIndex_ListWidget(d->list);
                 /* Update menu items. */
                 /* TODO: Some callback-based mechanism would be nice for updating menus right
                    before they open? */
@@ -1436,9 +1441,13 @@ static void draw_SidebarItem_(const iSidebarItem *d, iPaint *p, iRect itemRect,
                               const iListWidget *list) {
     const iSidebarWidget *sidebar = findParentClass_Widget(constAs_Widget(list),
                                                            &Class_SidebarWidget);
+    const iBool isMenuVisible = isVisible_Widget(sidebar->menu);
     const iBool isPressing   = isMouseDown_ListWidget(list);
-    const iBool isHover      = isHover_Widget(constAs_Widget(list)) &&
-                               constHoverItem_ListWidget(list) == d;
+    const iBool isHover      =
+            (!isMenuVisible &&
+            isHover_Widget(constAs_Widget(list)) &&
+            constHoverItem_ListWidget(list) == d) ||
+            (isMenuVisible && sidebar->contextItem == d);
     const int scrollBarWidth = scrollBarWidth_ListWidget(list);
 #if defined (iPlatformApple)
     const int blankWidth     = 0;
