@@ -23,6 +23,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "widget.h"
 
 #include "app.h"
+#include "periodic.h"
 #include "touch.h"
 #include "command.h"
 #include "paint.h"
@@ -98,6 +99,7 @@ static void aboutToBeDestroyed_Widget_(iWidget *d) {
     if (flags_Widget(d) & keepOnTop_WidgetFlag) {
         removeOne_PtrArray(onTop_Root(d->root), d);
     }
+    remove_Periodic(periodic_App(), d);
     if (isHover_Widget(d)) {
         get_Window()->hover = NULL;
     }
@@ -920,9 +922,12 @@ iBool dispatchEvent_Widget(iWidget *d, const SDL_Event *ev) {
 static iBool scrollOverflow_Widget_(iWidget *d, int delta) {
     iRect bounds = bounds_Widget(d);
     const iInt2 rootSize = size_Root(d->root);
-    const iRect winRect = safeRect_Root(d->root);
-    const int yTop = top_Rect(winRect);
-    const int yBottom = bottom_Rect(winRect);
+    const iRect winRect  = safeRect_Root(d->root);
+    const int   yTop     = top_Rect(winRect);
+    const int   yBottom  = bottom_Rect(winRect);
+    if (top_Rect(bounds) >= yTop && bottom_Rect(bounds) < yBottom) {
+        return iFalse; /* fits inside just fine */
+    }
     //const int safeBottom = rootSize.y - yBottom;
     bounds.pos.y += delta;
     const iRangei range = { bottom_Rect(winRect) - height_Rect(bounds), yTop };
