@@ -59,7 +59,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 static iWindow *theWindow_ = NULL;
 
-#if defined (iPlatformApple) || defined (iPlatformLinux)
+#if defined (iPlatformApple) || defined (iPlatformLinux) || defined (iPlatformOther)
 static float initialUiScale_ = 1.0f;
 #else
 static float initialUiScale_ = 1.1f;
@@ -515,15 +515,15 @@ void init_Window(iWindow *d, iRect rect) {
 
 void deinit_Window(iWindow *d) {
     iRecycle();
-    if (theWindow_ == d) {
-        theWindow_ = NULL;
-    }
     iForIndices(i, d->roots) {
         if (d->roots[i]) {
             setCurrent_Root(d->roots[i]);
             deinit_Root(d->roots[i]);
         }
     }
+    if (theWindow_ == d) {
+        theWindow_ = NULL;
+    }
     setCurrent_Root(NULL);
     delete_String(d->pendingSplitUrl);
     deinit_Text();
@@ -566,14 +566,18 @@ iRoot *otherRoot_Window(const iWindow *d, iRoot *root) {
     return root == d->roots[0] && d->roots[1] ? d->roots[1] : d->roots[0];
 }
 
-void invalidate_Window(iWindow *d) {
-    if (d && !d->isInvalidated) {
+static void invalidate_Window_(iWindow *d, iBool forced) {
+    if (d && (!d->isInvalidated || forced)) {
         d->isInvalidated = iTrue;
         resetFonts_Text();
         postCommand_App("theme.changed auto:1"); /* forces UI invalidation */
     }
 }
 
+void invalidate_Window(iWindow *d) {
+    invalidate_Window_(d, iFalse);
+}
+
 static iBool isNormalPlacement_Window_(const iWindow *d) {
     if (d->isDrawFrozen) return iFalse;
 #if defined (iPlatformApple)
@@ -741,14 +745,14 @@ static iBool handleWindowEvent_Window_(iWindow *d, const SDL_WindowEvent *ev) {
             return iTrue;
         case SDL_WINDOWEVENT_RESTORED:
             updateSize_Window_(d, iTrue);
-            invalidate_Window(d);
+            invalidate_Window_(d, iTrue);
             d->isMinimized = iFalse;
             postRefresh_App();
             return iTrue;
         case SDL_WINDOWEVENT_MINIMIZED:
             d->isMinimized = iTrue;
             return iTrue;
-#endif
+#endif /* defined (iPlatformDesktop) */
         case SDL_WINDOWEVENT_LEAVE:
             unhover_Widget();
             d->isMouseInside = iFalse;
@@ -769,10 +773,10 @@ static iBool handleWindowEvent_Window_(iWindow *d, const SDL_WindowEvent *ev) {
             d->focusGainedAt = SDL_GetTicks();
             setCapsLockDown_Keys(iFalse);
             postCommand_App("window.focus.gained");
+            d->isExposed = iTrue;
 #if defined (iPlatformMobile)
-            d->isExposed = iTrue; /* no expose event is sent, so now we know it's visible */
             /* Returned to foreground, may have lost buffered content. */
-            invalidate_Window_(d);
+            invalidate_Window_(d, iTrue);
             postCommand_App("window.unfreeze");
 #endif
             return iFalse;
@@ -854,6 +858,19 @@ iBool processEvent_Window(iWindow *d, const SDL_Event *ev) {
                 const iInt2 pos = coord_Window(d, event.button.x, event.button.y);
                 event.button.x = pos.x;
                 event.button.y = pos.y;
+                if (event.type == SDL_MOUSEBUTTONDOWN) {
+                    /* Button clicks will change keyroot. */
+                    if (numRoots_Window(d) > 1) {
+                        const iInt2 click = init_I2(event.button.x, event.button.y);
+                        iForIndices(i, d->roots) {
+                            iRoot *root = d->roots[i];
+                            if (root != d->keyRoot && contains_Rect(rect_Root(root), click)) {
+                                setKeyRoot_Window(d, root);
+                                break;
+                            }
+                        }
+                    }
+                }
             }
             const iWidget *oldHover = d->hover;
             iBool wasUsed = iFalse;
@@ -885,7 +902,9 @@ iBool processEvent_Window(iWindow *d, const SDL_Event *ev) {
                     wasUsed = dispatchEvent_Window(d, &paste);
                 }
                 if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_RIGHT) {
-                    postContextClick_Window(d, &event.button);
+                    if (postContextClick_Window(d, &event.button)) {
+                        wasUsed = iTrue;
+                    }
                 }
             }
             if (isMetricsChange_UserEvent(&event)) {
@@ -1008,7 +1027,10 @@ void draw_Window(iWindow *d) {
     /* Clear the window. The clear color is visible as a border around the window
        when the custom frame is being used. */ {
 #if defined (iPlatformAppleMobile)
-        const iColor back = get_Color(tmBackground_ColorId);
+        iColor back = get_Color(uiBackground_ColorId);
+        if (deviceType_App() == phone_AppDeviceType) {
+            /* Page background extends to safe area, so fill it completely. */
+            back = get_Color(tmBackground_ColorId);
 #else
         const iColor back = get_Color(gotFocus && d->place.snap != maximized_WindowSnap &&
                                               ~winFlags & SDL_WINDOW_FULLSCREEN_DESKTOP
@@ -1196,6 +1218,11 @@ void setSplitMode_Window(iWindow *d, int splitFlags) {
         setFreezeDraw_Window(d, iTrue);
         if (oldCount == 2 && splitMode == 0) {
             /* Keep references to the tabs of the second root. */
+            const iDocumentWidget *curPage = document_Root(d->keyRoot);
+            if (!curPage) {
+                /* All tabs closed on that side. */
+                curPage = document_Root(otherRoot_Window(d, d->keyRoot));
+            }
             iObjectList *tabs = listDocuments_App(d->roots[1]);
             iForEach(ObjectList, i, tabs) {
                 setRoot_Widget(i.object, d->roots[0]);
@@ -1213,6 +1240,7 @@ void setSplitMode_Window(iWindow *d, int splitFlags) {
             /* The last child is the [+] button for adding a tab. */
             moveTabButtonToEnd_Widget(findChild_Widget(docTabs, "newtab"));
             iRelease(tabs);
+            postCommandf_App("tabs.switch id:%s", cstr_String(id_Widget(constAs_Widget(curPage))));
         }
         else if (splitMode && oldCount == 1) {
             /* Add a second root. */
